@@ -4,9 +4,10 @@ import argparse
 import os
 from typing import Any, Callable, Sequence
 
-from click import secho
+from pdm.compat import Protocol
+from pdm.termui import UI
 
-from pdm._types import Protocol
+ui = UI()
 
 
 class ActionCallback(Protocol):
@@ -58,7 +59,7 @@ class ArgumentGroup(Option):
 
     def __init__(
         self,
-        name: str = None,
+        name: str,
         is_mutually_exclusive: bool = False,
         required: bool = None,
     ) -> None:
@@ -90,12 +91,47 @@ def deprecated(message: str, type_: type = str) -> Callable[[Any], Any]:
     """Prints deprecation message for the argument"""
 
     def wrapped_type(obj: Any) -> Any:
-        secho(f"DEPRECATED: {message}", fg="red", err=True)
+        ui.echo(f"DEPRECATED: {message}", style="red", err=True)
         return type_(obj)
 
     return wrapped_type
 
 
+def split_lists(separator: str) -> type[argparse.Action]:
+    """
+    Works the same as `append` except each argument
+    is considered a `separator`-separated list.
+    """
+
+    class SplitList(argparse.Action):
+        def __call__(
+            self,
+            parser: argparse.ArgumentParser,
+            args: argparse.Namespace,
+            values: Any,
+            option_string: str | None = None,
+        ) -> None:
+            if not isinstance(values, str):
+                return
+            split = getattr(args, self.dest) or []
+            split.extend(
+                value.strip() for value in values.split(separator) if value.strip()
+            )
+            setattr(args, self.dest, split)
+
+    return SplitList
+
+
+def from_splitted_env(name: str, separator: str) -> list[str] | None:
+    """
+    Parse a `separator`-separated list from a `name` environment variable if present.
+    """
+    value = os.getenv(name)
+    if not value:
+        return None
+    return [v.strip() for v in value.split(separator) if v.strip()] or None
+
+
 verbose_option = Option(
     "-v",
     "--verbose",
@@ -116,8 +152,7 @@ lockfile_option = Option(
     "-L",
     "--lockfile",
     default=os.getenv("PDM_LOCKFILE"),
-    help="Specify another lockfile path, or use `PDM_LOCKFILE` env variable. "
-    "Default: pdm.lock",
+    help="Specify another lockfile path. Default: pdm.lock. [env var: PDM_LOCKFILE]",
 )
 
 pep582_option = Option(
@@ -230,6 +265,18 @@ save_strategy_group.add_argument(
     help="Save minimum version specifiers",
 )
 
+skip_option = Option(
+    "-k",
+    "--skip",
+    dest="skip",
+    action=split_lists(","),
+    help="Skip some tasks and/or hooks by their comma-separated names."
+    " Can be supplied multiple times."
+    ' Use ":all" to skip all hooks.'
+    ' Use ":pre" and ":post" to skip all pre or post hooks.',
+    default=from_splitted_env("PDM_SKIP_HOOKS", ","),
+)
+
 update_strategy_group = ArgumentGroup("update_strategy", is_mutually_exclusive=True)
 update_strategy_group.add_argument(
     "--update-reuse",
@@ -272,10 +319,10 @@ global_option = Option(
 
 clean_group = ArgumentGroup("clean", is_mutually_exclusive=True)
 clean_group.add_argument(
-    "--clean", action="store_true", dest="clean", help="clean unused packages"
+    "--clean", action="store_true", help="clean packages not in the lockfile"
 )
 clean_group.add_argument(
-    "--no-clean", action="store_false", dest="clean", help="don't clean unused packages"
+    "--only-keep", action="store_true", help="only keep the selected packages"
 )
 
 sync_group = ArgumentGroup("sync", is_mutually_exclusive=True)
@@ -301,7 +348,7 @@ ignore_python_option = Option(
     "-I",
     "--ignore-python",
     action="store_true",
-    help="Ignore the Python path saved in the pdm.toml config",
+    help="Ignore the Python path saved in the .pdm.toml config",
 )
 
 prerelease_option = Option(
