@@ -1,8 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, List
-
-from pdm import termui
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     from pdm.models.candidates import Candidate
@@ -12,19 +10,23 @@ class PdmException(Exception):
     pass
 
 
+class PdmArgumentError(PdmException):
+    pass
+
+
 class PdmUsageError(PdmException):
     pass
 
 
-class RequirementError(PdmException, ValueError):
+class RequirementError(PdmUsageError, ValueError):
     pass
 
 
-class InvalidPyVersion(PdmException, ValueError):
+class PublishError(PdmUsageError):
     pass
 
 
-class CorruptedCacheError(PdmException):
+class InvalidPyVersion(PdmUsageError, ValueError):
     pass
 
 
@@ -35,15 +37,14 @@ class CandidateNotFound(PdmException):
 class CandidateInfoNotFound(PdmException):
     def __init__(self, candidate: Candidate) -> None:
         message = (
-            "No metadata information is available for "
-            f"{termui.green(str(candidate))}."
+            "No metadata information is available for " f"[green]{str(candidate)}[/]."
         )
         self.candidate = candidate
         super().__init__(message)
 
 
 class ExtrasWarning(UserWarning):
-    def __init__(self, project_name: str, extras: List[str]) -> None:
+    def __init__(self, project_name: str, extras: list[str]) -> None:
         super().__init__(f"Extras not found for {project_name}: [{','.join(extras)}]")
         self.extras = tuple(extras)
 
@@ -60,12 +61,12 @@ class UninstallError(PdmException):
     pass
 
 
-class NoConfigError(PdmException, KeyError):
+class NoConfigError(PdmUsageError, KeyError):
     def __init__(self, key: str) -> None:
-        super().__init__("No such config item: {}".format(key))
+        super().__init__(f"No such config item: {key}")
 
 
-class NoPythonVersion(PdmException):
+class NoPythonVersion(PdmUsageError):
     pass
 
 
