@@ -6,15 +6,17 @@ import os
 import shutil
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Iterable, Type, TypeVar, cast
+from typing import TYPE_CHECKING, Iterable, TypeVar, cast
 
 from pdm import termui
-from pdm._types import Distribution
 from pdm.exceptions import UninstallError
 from pdm.installers.packages import CachedPackage
-from pdm.models.environment import Environment
 from pdm.utils import is_egg_link, is_path_relative_to
 
+if TYPE_CHECKING:
+    from pdm.compat import Distribution
+    from pdm.models.environment import Environment
+
 _T = TypeVar("_T", bound="BaseRemovePaths")
 
 
@@ -93,8 +95,7 @@ def _cache_file_from_source(py_file: str) -> Iterable[str]:
         yield py2_cache
     parent, base = os.path.split(py_file)
     cache_dir = os.path.join(parent, "__pycache__")
-    for path in glob.glob(os.path.join(cache_dir, base[:-3] + ".*.pyc")):
-        yield path
+    yield from glob.glob(os.path.join(cache_dir, base[:-3] + ".*.pyc"))
 
 
 def _get_file_root(path: str, base: str) -> str | None:
@@ -128,7 +129,7 @@ class BaseRemovePaths(abc.ABC):
         """Roll back the removal operations"""
 
     @classmethod
-    def from_dist(cls: Type[_T], dist: Distribution, environment: Environment) -> _T:
+    def from_dist(cls: type[_T], dist: Distribution, environment: Environment) -> _T:
         """Create an instance from the distribution"""
         scheme = environment.get_paths()
         instance = cls(dist, environment)
@@ -266,7 +267,7 @@ class StashedRemovePaths(BaseRemovePaths):
         self._stashed.clear()
         self._saved_pth = None
         if self.refer_to:
-            termui.logger.debug("Unlink from cached package %s", self.refer_to)
+            termui.logger.info("Unlink from cached package %s", self.refer_to)
             CachedPackage(self.refer_to).remove_referrer(os.path.dirname(self.refer_to))
             self.refer_to = None
 
