@@ -7,21 +7,22 @@ import re
 import shutil
 import sys
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Iterable, Type, cast
+from typing import TYPE_CHECKING, Any, Iterable, cast
 from urllib.parse import urlparse
 
+import platformdirs
 import tomlkit
 from findpython import Finder
+from tomlkit.items import Array
 
 from pdm import termui
 from pdm._types import Source
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
-from pdm.models import pip_shims
-from pdm.models.caches import CandidateInfoCache, HashCache
-from pdm.models.candidates import Candidate
+from pdm.models.caches import CandidateInfoCache, HashCache, WheelCache
+from pdm.models.candidates import Candidate, make_candidate
 from pdm.models.environment import Environment, GlobalEnvironment
 from pdm.models.python import PythonInfo
-from pdm.models.repositories import BaseRepository, LockedRepository, PyPIRepository
+from pdm.models.repositories import BaseRepository, LockedRepository
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import PySpecSet, get_specifier
 from pdm.project.config import Config
@@ -34,15 +35,15 @@ from pdm.utils import (
     expand_env_vars_in_auth,
     find_project_root,
     find_python_in_path,
-    get_in_project_venv_python,
     get_venv_like_prefix,
     normalize_name,
+    path_to_url,
 )
 
 if TYPE_CHECKING:
     from resolvelib.reporters import BaseReporter
+    from rich.status import Status
 
-    import halo
     from pdm.core import Core
     from pdm.resolver.providers import BaseProvider
 
@@ -62,7 +63,7 @@ class Project:
 
     PYPROJECT_FILENAME = "pyproject.toml"
     DEPENDENCIES_RE = re.compile(r"(?:(.+?)-)?dependencies")
-    LOCKFILE_VERSION = "3.1"
+    LOCKFILE_VERSION = "4.0"
 
     def __init__(
         self,
@@ -78,7 +79,7 @@ class Project:
         self.core = core
 
         if global_config is None:
-            global_config = Path.home() / ".pdm/config.toml"
+            global_config = platformdirs.user_config_path("pdm") / "config.toml"
         self.global_config = Config(Path(global_config), is_global=True)
         global_project = Path(self.global_config["global_project.path"])
 
@@ -93,13 +94,14 @@ class Project:
             and root_path is None
             and self.global_config["global_project.fallback"]
         ):
-            self.core.ui.echo(
-                "Project is not found, fallback to the global project",
-                fg="yellow",
-                err=True,
-            )
             root_path = global_project
             is_global = True
+            if self.global_config["global_project.fallback_verbose"]:
+                self.core.ui.echo(
+                    "Project is not found, fallback to the global project",
+                    style="yellow",
+                    err=True,
+                )
 
         self.root = Path(root_path or "").absolute()
         self.is_global = is_global
@@ -124,9 +126,12 @@ class Project:
 
     @property
     def pyproject(self) -> dict | None:
-        if not self._pyproject and self.pyproject_file.exists():
-            data = tomlkit.parse(self.pyproject_file.read_text("utf-8"))
-            self._pyproject = cast(dict, data)
+        if not self._pyproject:
+            if self.pyproject_file.exists():
+                data = tomlkit.parse(self.pyproject_file.read_text("utf-8"))
+                self._pyproject = cast(dict, data)
+            else:
+                self._pyproject = cast(dict, tomlkit.document())
         return self._pyproject
 
     @pyproject.setter
@@ -197,35 +202,54 @@ class Project:
 
     def resolve_interpreter(self) -> PythonInfo:
         """Get the Python interpreter path."""
+        from pdm.cli.commands.venv.utils import get_venv_python, iter_venvs
+
+        def match_version(python: PythonInfo) -> bool:
+            return python.valid and self.python_requires.contains(python.version, True)
+
+        def note(message: str) -> None:
+            if not self.is_global:
+                self.core.ui.echo(message, style="yellow", err=True)
+
         config = self.config
         if config.get("python.path") and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
             saved_path = config["python.path"]
             python = PythonInfo.from_path(saved_path)
-            if python.valid and self.python_requires.contains(
-                str(python.version), True
-            ):
+            if match_version(python):
                 return python
             self.project_config.pop("python.path", None)
-        if os.name == "nt":
-            suffix = ".exe"
-            scripts = "Scripts"
-        else:
-            suffix = ""
-            scripts = "bin"
 
-        # Resolve virtual environments from env-vars
-        virtual_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
-        if config["python.use_venv"] and virtual_env:
-            python = PythonInfo.from_path(
-                os.path.join(virtual_env, scripts, f"python{suffix}")
-            )
-            if python.valid:
-                return python
+        if config.get("python.use_venv") and not self.is_global:
+            # Resolve virtual environments from env-vars
+            venv_in_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
+            if venv_in_env:
+                python = PythonInfo.from_path(get_venv_python(Path(venv_in_env)))
+                if match_version(python):
+                    note(
+                        f"Inside an active virtualenv [green]{venv_in_env}[/], "
+                        "reuse it."
+                    )
+                    return python
+            # otherwise, get a venv associated with the project
+            for _, venv in iter_venvs(self):
+                python = PythonInfo.from_path(get_venv_python(venv))
+                if match_version(python):
+                    note(f"Virtualenv [green]{venv}[/] is reused.")
+                    self.python = python
+                    return python
+
+            if not self.root.joinpath("__pypackages__").exists():
+                note(
+                    "python.use_venv is on, creating a virtualenv for this project"
+                    "..."
+                )
+                venv = self._create_virtualenv()
+                self.python = PythonInfo.from_path(get_venv_python(venv))
+                return self.python
 
         for py_version in self.find_interpreters():
-            if py_version.valid and self.python_requires.contains(
-                str(py_version.version), True
-            ):
+            if match_version(py_version):
+                note("[green]__pypackages__[/] is detected, using the PEP 582 mode")
                 self.python = py_version
                 return py_version
 
@@ -237,18 +261,33 @@ class Project:
 
     def get_environment(self) -> Environment:
         """Get the environment selected by this project"""
+
         if self.is_global:
             env = GlobalEnvironment(self)
             # Rewrite global project's python requires to be
             # compatible with the exact version
             env.python_requires = PySpecSet(f"=={self.python.version}")
             return env
-        if self.config["python.use_venv"] and get_venv_like_prefix(
-            self.python.executable
-        ):
-            # Only recognize venv created by python -m venv and virtualenv>20
-            return GlobalEnvironment(self)
-        return Environment(self)
+
+        return (
+            GlobalEnvironment(self)
+            if self.config["python.use_venv"]
+            and get_venv_like_prefix(self.python.executable) is not None
+            else Environment(self)
+        )
+
+    def _create_virtualenv(self) -> Path:
+        from pdm.cli.commands.venv.backends import BACKENDS
+
+        backend: str = self.config["venv.backend"]
+        venv_backend = BACKENDS[backend](self, None)
+        path = venv_backend.create(
+            force=True, in_project=self.config["venv.in_project"]
+        )
+        self.core.ui.echo(
+            f"Virtualenv is created successfully at [green]{path}[/]", err=True
+        )
+        return path
 
     @property
     def environment(self) -> Environment:
@@ -266,9 +305,11 @@ class Project:
 
     def get_dependencies(self, group: str | None = None) -> dict[str, Requirement]:
         metadata = self.meta
+        group = group or "default"
         optional_dependencies = metadata.get("optional-dependencies", {})
         dev_dependencies = self.tool_settings.get("dev-dependencies", {})
-        if group in (None, "default"):
+        in_metadata = group == "default" or group in optional_dependencies
+        if group == "default":
             deps = metadata.get("dependencies", [])
         else:
             if group in optional_dependencies and group in dev_dependencies:
@@ -276,7 +317,7 @@ class Project:
                     f"The {group} group exists in both [optional-dependencies] "
                     "and [dev-dependencies], the former is taken.",
                     err=True,
-                    fg="yellow",
+                    style="yellow",
                 )
             if group in optional_dependencies:
                 deps = optional_dependencies[group]
@@ -288,6 +329,15 @@ class Project:
         with cd(self.root):
             for line in deps:
                 if line.startswith("-e "):
+                    if in_metadata:
+                        self.core.ui.echo(
+                            f"WARNING: Skipping editable dependency [b]{line}[/] in the"
+                            r" [green]\[project][/] table. Please move it to the "
+                            r"[green]\[tool.pdm.dev-dependencies][/] table",
+                            err=True,
+                            style="yellow",
+                        )
+                        continue
                     req = parse_requirement(line[3:].strip(), True)
                 else:
                     req = parse_requirement(line)
@@ -332,18 +382,23 @@ class Project:
     def allow_prereleases(self) -> bool | None:
         return self.tool_settings.get("allow_prereleases")
 
+    @property
+    def default_source(self) -> Source:
+        """Get the default source of from the pypi setting"""
+        return cast(
+            "Source",
+            {
+                "url": self.config["pypi.url"],
+                "verify_ssl": self.config["pypi.verify_ssl"],
+                "name": "pypi",
+            },
+        )
+
     @property
     def sources(self) -> list[Source]:
         sources = list(self.tool_settings.get("source", []))
         if all(source.get("name") != "pypi" for source in sources):
-            sources.insert(
-                0,
-                {
-                    "url": self.config["pypi.url"],
-                    "verify_ssl": self.config["pypi.verify_ssl"],
-                    "name": "pypi",
-                },
-            )
+            sources.insert(0, self.default_source)
         expanded_sources: list[Source] = [
             Source(
                 url=expand_env_vars_in_auth(s["url"]),
@@ -355,10 +410,10 @@ class Project:
         ]
         return expanded_sources
 
-    def get_repository(self, cls: Type[BaseRepository] | None = None) -> BaseRepository:
+    def get_repository(self, cls: type[BaseRepository] | None = None) -> BaseRepository:
         """Get the repository object"""
         if cls is None:
-            cls = PyPIRepository
+            cls = self.core.repository_class
         sources = self.sources or []
         return cls(sources, self.environment)
 
@@ -393,23 +448,27 @@ class Project:
             ReusePinProvider,
         )
 
-        repository = self.get_repository(cls=self.core.repository_class)
+        repository = self.get_repository()
         allow_prereleases = self.allow_prereleases
         overrides = {
             normalize_name(k): v
             for k, v in self.tool_settings.get("overrides", {}).items()
         }
-        if strategy != "all" and not self.is_lockfile_compatible():
-            self.core.ui.echo(
-                "Updating the whole lock file as it is not compatible with PDM",
-                fg="yellow",
-                err=True,
-            )
-            strategy = "all"
-        if not for_install and strategy == "all":
-            return BaseProvider(repository, allow_prereleases, overrides)
+        locked_repository: LockedRepository | None = None
+        if strategy != "all" or for_install:
+            try:
+                locked_repository = self.locked_repository
+            except Exception:
+                if for_install:
+                    raise
+                self.core.ui.echo(
+                    "Unable to reuse the lock file as it is not compatible with PDM",
+                    style="yellow",
+                    err=True,
+                )
 
-        locked_repository = self.locked_repository
+        if locked_repository is None:
+            return BaseProvider(repository, allow_prereleases, overrides)
         if for_install:
             return BaseProvider(locked_repository, allow_prereleases, overrides)
         provider_class = (
@@ -428,7 +487,7 @@ class Project:
         self,
         requirements: list[Requirement],
         tracked_names: Iterable[str] | None = None,
-        spinner: halo.Halo | termui.DummySpinner | None = None,
+        spinner: Status | termui.DummySpinner | None = None,
     ) -> BaseReporter:
         """Return the reporter object to construct a resolver.
 
@@ -455,15 +514,15 @@ class Project:
             with atomic_open_for_write(self.lockfile_file) as fp:
                 tomlkit.dump(toml_data, fp)  # type: ignore
             if show_message:
-                self.core.ui.echo(f"Changes are written to {termui.green('pdm.lock')}.")
+                self.core.ui.echo("Changes are written to [green]pdm.lock[/].")
             self._lockfile = None
         else:
             self._lockfile = toml_data
 
     def make_self_candidate(self, editable: bool = True) -> Candidate:
-        req = parse_requirement(pip_shims.path_to_url(self.root.as_posix()), editable)
+        req = parse_requirement(path_to_url(self.root.as_posix()), editable)
         req.name = self.meta.name
-        return Candidate(req, name=self.meta.name, version=self.meta.version)
+        return make_candidate(req, name=self.meta.name, version=self.meta.version)
 
     def get_content_hash(self, algo: str = "md5") -> str:
         # Only calculate sources and dependencies groups. Otherwise lock file is
@@ -510,15 +569,14 @@ class Project:
         """Get the dependencies array in the pyproject.toml"""
         if group == "default":
             return self.meta.setdefault("dependencies", [])
-        else:
-            deps_dict = {
-                False: self.meta.setdefault("optional-dependencies", {}),
-                True: self.tool_settings.setdefault("dev-dependencies", {}),
-            }
-            for deps in deps_dict.values():
-                if group in deps:
-                    return deps[group]
-            return deps_dict[dev].setdefault(group, [])
+        deps_dict = {
+            False: self.meta.setdefault("optional-dependencies", {}),
+            True: self.tool_settings.setdefault("dev-dependencies", {}),
+        }
+        for deps in deps_dict.values():
+            if group in deps:
+                return deps[group]
+        return deps_dict[dev].setdefault(group, [])
 
     def add_dependencies(
         self,
@@ -526,14 +584,12 @@ class Project:
         to_group: str = "default",
         dev: bool = False,
         show_message: bool = True,
-        replace_editable: bool = False,
     ) -> None:
-        deps = self.get_pyproject_dependencies(to_group, dev).multiline(  # type: ignore
-            True
-        )
+        deps = cast(Array, self.get_pyproject_dependencies(to_group, dev))
+        deps.multiline(True)
         for _, dep in requirements.items():
             matched_index = next(
-                (i for i, r in enumerate(deps) if dep.matches(r, not replace_editable)),
+                (i for i, r in enumerate(deps) if dep.matches(r)),
                 None,
             )
             if matched_index is None:
@@ -549,19 +605,14 @@ class Project:
         ) as f:
             tomlkit.dump(self.pyproject, f)  # type: ignore
         if show_message:
-            self.core.ui.echo(
-                f"Changes are written to {termui.green('pyproject.toml')}."
-            )
+            self.core.ui.echo("Changes are written to [green]pyproject.toml[/].")
         self._pyproject = None
 
     @property
     def meta(self) -> Metadata:
         if not self.pyproject:
             self.pyproject = {"project": tomlkit.table()}
-        m = Metadata(self.pyproject_file, False)
-        m._metadata = self.pyproject.get("project", {})
-        m._tool_settings = self.tool_settings
-        return m
+        return Metadata(self.root, self.pyproject)
 
     def init_global_project(self) -> None:
         if not self.is_global:
@@ -582,16 +633,17 @@ dependencies = ["pip", "setuptools", "wheel"]
 
     def cache(self, name: str) -> Path:
         path = self.cache_dir / name
-        path.mkdir(parents=True, exist_ok=True)
+        try:
+            path.mkdir(parents=True, exist_ok=True)
+        except OSError:
+            # The path could be not accessible
+            pass
         return path
 
-    def make_wheel_cache(self) -> pip_shims.WheelCache:
-        return pip_shims.WheelCache(
-            self.cache_dir.as_posix(), pip_shims.FormatControl(set(), set())
-        )
+    def make_wheel_cache(self) -> WheelCache:
+        return WheelCache(self.cache("wheels"))
 
     def make_candidate_info_cache(self) -> CandidateInfoCache:
-
         python_hash = hashlib.sha1(
             str(self.environment.python_requires).encode()
         ).hexdigest()
@@ -599,7 +651,7 @@ dependencies = ["pip", "setuptools", "wheel"]
         return CandidateInfoCache(self.cache("metadata") / file_name)
 
     def make_hash_cache(self) -> HashCache:
-        return HashCache(directory=self.cache("hashes").as_posix())
+        return HashCache(directory=self.cache("hashes"))
 
     def find_interpreters(self, python_spec: str | None = None) -> Iterable[PythonInfo]:
         """Return an iterable of interpreter paths that matches the given specifier,
@@ -621,10 +673,6 @@ dependencies = ["pip", "setuptools", "wheel"]
                     yield PythonInfo.from_path(pyenv_shim)
                 elif os.path.exists(pyenv_shim.replace("python3", "python")):
                     yield PythonInfo.from_path(pyenv_shim.replace("python3", "python"))
-            if config.get("python.use_venv"):
-                python = get_in_project_venv_python(self.root)
-                if python:
-                    yield PythonInfo.from_path(python)
             python = shutil.which("python")
             if python:
                 yield PythonInfo.from_path(python)
@@ -641,9 +689,18 @@ dependencies = ["pip", "setuptools", "wheel"]
                         yield PythonInfo.from_path(python)
                 return
             args = [int(v) for v in python_spec.split(".") if v != ""]
-        finder = Finder(resolve_symlinks=True)
+        finder = self._get_python_finder()
         for entry in finder.find_all(*args):
             yield PythonInfo(entry)
         if not python_spec:
+            # Lastly, return the host Python as well
             this_python = getattr(sys, "_base_executable", sys.executable)
             yield PythonInfo.from_path(this_python)
+
+    def _get_python_finder(self) -> Finder:
+        from pdm.cli.commands.venv.utils import VenvProvider
+
+        finder = Finder(resolve_symlinks=True)
+        if self.config["python.use_venv"]:
+            finder._providers.insert(0, VenvProvider(self))
+        return finder
